
/*
** Copyright (C) 2003-2006 Herbert Poetzl
**
** V0.01 test for syscall
** V0.02 new scheduler args
** V0.03 force flag and help
**
*/

#define VERSION "V0.03"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
 
#include "syscall.h"
#include "vserver.h"
#include "switch.h"
#include "sched_cmd.h"

static	char	err_flag = 0;

static	char	opt_vers = 0;
static	char	opt_sched = 0;
static	char	opt_idle = 0;
static	char	opt_force = 0;

static	char *	cmd_name = NULL;

static	int 	num_xid = 0;

static	struct 	vcmd_set_sched_v4 data;


#define OPTIONS "+fhin:u:x:I:M:P:R:S:T:V"

int	main(int argc, char *argv[])
{
        extern int optind;
        extern char *optarg;
        char c, errflg = 0;
	int r;


        cmd_name = argv[0];
        while ((c = getopt(argc, argv, OPTIONS)) != EOF) {
            switch (c) {
            case 'h':
                fprintf(stderr,
                    "This is %s " VERSION "\n"
                    "options are:\n"
                    "-f        force bucket update\n"
                    "-h        print this help message\n"
                    "-x <num>  context id\n"
                    "-u <num>  cpu id\n"
                    "-n <num>  bucket id\n"
		    "-i        idle_time flag\n"
                    "-I <num>  set fill interval to <num>\n"
                    "-R <num>  set fill rate to <num>\n"
                    "-M <num>  set unhold minumum to <num>\n"
                    "-S <num>  set bucket size to <num>\n"
                    "-T <num>  set token number to <num>\n"
                    "-P <num>  set priority base to <num>\n"
                    "-V        verify interface version\n"
                    "--        end of options\n\n"
                    "note: -I/R has different meaning before and\n"
		    "       after -i, and can be given twice.\n"
                    ,cmd_name);
                exit(0);
                break;
            case 'f':
		data.set_mask |= VXSM_FORCE;
		opt_force = 1;
                break;
            case 'i':
		data.set_mask |= VXSM_IDLE_TIME;
		opt_idle = 1;
                break;
            case 'n':
                data.bucket_id = atol(optarg);
		data.set_mask |= VXSM_BUCKET_ID;
                break;
            case 'u':
                data.cpu_id = atol(optarg);
		data.set_mask |= VXSM_CPU_ID;
                break;
            case 'x':
                num_xid = atol(optarg);
                break;
            case 'I':
                data.interval = atol(optarg);
		data.set_mask |= (opt_idle) ?
			VXSM_INTERVAL2 : VXSM_INTERVAL;
                opt_sched = 1;
                break;
            case 'M':
                data.tokens_min = atol(optarg);
		data.set_mask |= VXSM_TOKENS_MIN;
                opt_sched = 1;
                break;
            case 'P':
                data.prio_bias = atol(optarg);
		data.set_mask |= VXSM_PRIO_BIAS;
                opt_sched = 1;
                break;
            case 'R':
                data.fill_rate = atol(optarg);
		data.set_mask |= (opt_idle) ?
			VXSM_FILL_RATE2 : VXSM_FILL_RATE;
                opt_sched = 1;
                break;
            case 'S':
                data.tokens_max = atol(optarg);
		data.set_mask |= VXSM_TOKENS_MAX;
                opt_sched = 1;
                break;
            case 'T':
                data.tokens = atol(optarg);
		data.set_mask |= VXSM_TOKENS;
                opt_sched = 1;
                break;
            case 'V':
                opt_vers = 1;
                break;
            case '?':
            default:
                errflg++;
                break;
            }
        }
        if (errflg) {
            fprintf(stderr, 
                "Usage: %s -[" OPTIONS "]\n"
                "%s -h for help.\n",
                cmd_name, cmd_name);
            exit(2);
        }

	if (opt_vers) {
	    r = vserver(VCMD_get_version, 0, NULL);
	    if (r<0)
	    	perror("vc_get_version");
    	    else 
	    	printf("version: %04x:%04x\n", 
		    (r>>16)&0xFFFF, r&0xFFFF);
	}

	if (opt_sched) {
	    r = vserver(VCMD_set_sched, num_xid, &data);
	    if (r)
	    	perror("vc_set_sched");
	}
	
	exit((err_flag)?1:0);
}

